pub mod errors;
pub mod maven;
mod serde;
mod util;
pub mod version;

pub use version::{Version, VersionRestriction};

#[cfg(test)]
mod tests {
    use crate::maven::{
        qualifier::Qualifier, restriction::Restriction, version::ComparableVersion,
    };
    #[test]
    fn qualifiers() {
        let q1 = Qualifier::new("".to_string());
        let q2 = Qualifier::new("foo".to_string());
        let q3 = Qualifier::new("bar".to_string());
        let q4 = Qualifier::new("final".to_string());
        let q5 = Qualifier::new("beta".to_string());
        assert!(q1 < q2);
        assert!(q2 > q3);
        assert!(q1 == q4);
        assert!(q1 > q5);
    }
    #[test]
    fn version_parse() {
        let v1 = ComparableVersion::parse("1.0.0".to_string()).unwrap();
        let v2 = ComparableVersion::parse("1.ga".to_string()).unwrap();
        let v3 = ComparableVersion::parse("1.final".to_string()).unwrap();
        let v4 = ComparableVersion::parse("1.0".to_string()).unwrap();
        let v5 = ComparableVersion::parse("1.".to_string()).unwrap();
        let v6 = ComparableVersion::parse("1-".to_string()).unwrap();
        let v7 = ComparableVersion::parse("1.0.0-foo.0.0".to_string()).unwrap();
        let v8 = ComparableVersion::parse("1.0.0-0.0.0".to_string()).unwrap();
        assert_eq!(v1.parsed_string(), "[1]".to_string());
        assert_eq!(v2.parsed_string(), "[1]".to_string());
        assert_eq!(v3.parsed_string(), "[1]".to_string());
        assert_eq!(v4.parsed_string(), "[1]".to_string());
        assert_eq!(v5.parsed_string(), "[1]".to_string());
        assert_eq!(v6.parsed_string(), "[1]".to_string());
        assert_eq!(v7.parsed_string(), "[1, [foo]]".to_string());
        assert_eq!(v8.parsed_string(), "[1]".to_string());
    }
    #[test]
    fn version_cmp() {
        let v1 = ComparableVersion::parse("1".to_string()).unwrap();
        let v2 = ComparableVersion::parse("1.1".to_string()).unwrap();
        let v3 = ComparableVersion::parse("1-snapshot".to_string()).unwrap();
        let v4 = ComparableVersion::parse("1-sp".to_string()).unwrap();
        let v5 = ComparableVersion::parse("1-foo2".to_string()).unwrap();
        let v6 = ComparableVersion::parse("1-foo10".to_string()).unwrap();
        let v7 = ComparableVersion::parse("1.foo".to_string()).unwrap();
        let v8 = ComparableVersion::parse("1-foo".to_string()).unwrap();
        let v9 = ComparableVersion::parse("1-1".to_string()).unwrap();
        let v10 = ComparableVersion::parse("1.ga".to_string()).unwrap();
        let v11 = ComparableVersion::parse("1-ga".to_string()).unwrap();
        let v12 = ComparableVersion::parse("1.0".to_string()).unwrap();
        let v13 = ComparableVersion::parse("1-0".to_string()).unwrap();
        let v14 = ComparableVersion::parse("1-sp.1".to_string()).unwrap();
        let v15 = ComparableVersion::parse("1-ga.1".to_string()).unwrap();
        let v16 = ComparableVersion::parse("1-sp-1".to_string()).unwrap();
        let v17 = ComparableVersion::parse("1-ga-1".to_string()).unwrap();
        let v18 = ComparableVersion::parse("1-a1".to_string()).unwrap();
        let v19 = ComparableVersion::parse("1-alpha-1".to_string()).unwrap();
        //number padding
        assert!(v1 < v2);
        //qualifier padding
        assert!(v3 < v1);
        assert!(v1 < v4);
        //correctly automatically "switching" to numeric order
        assert!(v5 < v6);
        //dot and hyphen order
        assert!(v7 < v8);
        assert!(v8 < v9);
        assert!(v9 < v2);
        //removing of trailing "null" values
        assert!(v1 == v10);
        assert!(v1 == v11);
        assert!(v1 == v12);
        assert!(v1 == v13);
        //compare ga and sp
        assert!(v4 > v11);
        assert!(v14 > v15);
        //trailing "null" values at each hyphen
        assert!(v16 < v17);
        assert!(v17 < v9); //Test not in sync with spec but with official implementation
                           //alpha qualifier
        assert!(v18 == v19);
    }
    #[test]
    fn restrictions() {
        let restr1 = Restriction::parse("[1.0,1.2]".to_string()).unwrap();
        let restr2 = Restriction::parse("[1.1]".to_string()).unwrap();
        let restr3 = Restriction::parse("[1.0,1.2.1)".to_string()).unwrap();
        let restr4 = Restriction::parse("[1.0,)".to_string()).unwrap();
        let restr5 = Restriction::parse("(,1.2]".to_string()).unwrap();
        let restr6 = Restriction::parse("[1.2,1.0]".to_string());
        let v1 = ComparableVersion::parse("1.2.1".to_string()).unwrap();
        let v2 = ComparableVersion::parse("1.1".to_string()).unwrap();
        let v3 = ComparableVersion::parse("0.1".to_string()).unwrap();
        let v4 = ComparableVersion::parse("1.2".to_string()).unwrap();
        assert!(!restr1.contains_version(&v1));
        assert!(restr1.contains_version(&v2));
        assert!(!restr1.contains_version(&v3));
        assert!(restr1.contains_version(&v4));
        assert!(restr2.contains_version(&v2));
        assert!(!restr2.contains_version(&v4));
        assert!(restr3.contains_version(&v4));
        assert!(!restr3.contains_version(&v1));
        assert!(restr4.contains_version(&v1));
        assert!(!restr4.contains_version(&v3));
        assert!(!restr5.contains_version(&v1));
        assert!(restr5.contains_version(&v3));
        assert!(restr6.is_err());
    }
}
