use crate::{maven::qualifier::Qualifier, util::StringTools};
use std::slice::Iter;
#[derive(Debug)]
pub enum Item {
    // impl isnull
    String(Qualifier),
    List(ListItem),
    Integer(u64),
}
impl Item {
    pub fn is_list(&self) -> bool {
        matches!(self, Item::List(_))
    }
    pub fn ordinal(&self) -> u8 {
        match self {
            Item::String(_) => 0,
            Item::List(_) => 1,
            Item::Integer(_) => 2,
        }
    }
    pub fn zero() -> Self {
        Item::Integer(0)
    }
    pub fn new_int(value: String) -> Option<Self> {
        let number = value.parse::<u64>();
        match number {
            Ok(num) => Some(Item::Integer(num)),
            Err(_) => None,
        }
    }
    pub fn new_string(value: String) -> Self {
        Self::String(Qualifier::new(value))
    }
    pub fn parse_item(value: String) -> Self {
        Self::new_int(value.clone()).unwrap_or_else(|| Self::new_string(value))
    }
    pub fn parse_list(value: String) -> Self {
        let mut list = ListItem(vec![]);
        let mut cache = String::new();
        for (i, c) in value.chars().enumerate() {
            if c == '-' || c == '.' {
                list.add(Item::parse_item(cache.clone()));
                cache.clear();
                if c == '-' {
                    let sublist = value.substring(i + 1, value.length());
                    list.add(Item::parse_list(sublist));
                    break;
                }
            } else {
                cache.push(c);
            }
        }
        if !cache.is_empty() {
            list.add(Item::parse_item(cache));
        }
        list.normalize();
        Item::List(list)
    }
    pub fn is_null(&self) -> bool {
        match self {
            Item::Integer(int) => *int == 0,
            Item::List(lst) => lst.is_empty(),
            Item::String(string) => string.is_release(),
        }
    }
}
impl ToString for Item {
    fn to_string(&self) -> String {
        match self {
            Item::String(s) => s.to_string(),
            Item::Integer(int) => int.to_string(),
            Item::List(l) => {
                let mut string = "[".to_string();
                for (i, item) in l.iter().enumerate() {
                    if i > 0 {
                        string.push_str(", ")
                    }
                    string.push_str(item.to_string().as_str());
                }
                string.push(']');
                string
            }
        }
    }
}
#[derive(Debug)]
pub struct ListItem(pub Vec<Item>);
impl ListItem {
    pub fn normalize(&mut self) {
        let mut i = self.len();
        while i > 0 {
            let last_item = self.0.get(i - 1).unwrap();
            if last_item.is_null() {
                self.0.remove(i - 1);
            } else if !last_item.is_list() {
                break;
            }
            i -= 1;
        }
    }
    pub fn add(&mut self, item: Item) {
        self.0.push(item);
    }
    pub fn len(&self) -> usize {
        self.0.len()
    }
    pub fn iter(&self) -> Iter<Item> {
        self.0.iter()
    }
    pub fn is_empty(&self) -> bool {
        self.0.is_empty()
    }
}
impl Clone for Item {
    fn clone(&self) -> Self {
        match self {
            Self::String(arg0) => Self::String(arg0.clone()),
            Self::List(arg0) => Self::List(arg0.clone()),
            Self::Integer(arg0) => Self::Integer(*arg0),
        }
    }
}
impl Clone for ListItem {
    fn clone(&self) -> Self {
        Self(self.0.clone())
    }
}
