use crate::{
    errors::{Error, Result},
    util::StringTools,
};

use super::version::{ComparableVersion, VersionOption};
#[derive(Debug, Clone)]
pub struct VersionRange {
    pub(crate) restrictions: Vec<Restriction>,
    pub(crate) recommended_version: Option<ComparableVersion>,
}
#[derive(Debug, Clone)]
pub struct Restriction {
    pub lower_bound: Option<ComparableVersion>,
    pub lower_bound_inclusive: bool,
    pub upper_bound: Option<ComparableVersion>,
    pub upper_bound_inclusive: bool,
}
impl Restriction {
    pub(crate) fn contains_version(&self, version: &ComparableVersion) -> bool {
        let lower_satisfied = match &self.lower_bound {
            Some(l) => {
                if self.lower_bound_inclusive {
                    l <= version
                } else {
                    l < version
                }
            }
            None => true,
        };
        let upper_satisfied = match &self.upper_bound {
            Some(u) => {
                if self.upper_bound_inclusive {
                    u >= version
                } else {
                    u > version
                }
            }
            None => true,
        };
        lower_satisfied && upper_satisfied
    }
    pub(crate) fn unrestricted() -> Self {
        Self {
            lower_bound: None,
            lower_bound_inclusive: false,
            upper_bound: None,
            upper_bound_inclusive: false,
        }
    }
    pub(crate) fn parse(spec: String) -> Result<Self> {
        let lower_bound_inclusive = spec.starts_with('[');
        let upper_bound_inclusive = spec.ends_with(']');
        let process = spec.substring(1, spec.length() - 1);
        let index = process.index_of(',');
        match index {
            Some(i) => {
                let lower_version = process.substring(0, i);
                let upper_version = process.substring(i + 1, process.length());
                let lower_version = if lower_version.length() == 0 {
                    None
                } else {
                    Some(ComparableVersion::parse(process.substring(0, i))?)
                };
                let upper_version = if upper_version.length() == 0 {
                    None
                } else {
                    Some(ComparableVersion::parse(
                        process.substring(i + 1, process.length()),
                    )?)
                };
                let res = Self {
                    lower_bound: lower_version.clone(),
                    lower_bound_inclusive,
                    upper_bound: upper_version.clone(),
                    upper_bound_inclusive,
                };
                if let (Some(l), Some(u)) = (lower_version, upper_version) {
                    if l > u {
                        return Err(Error::IllegalBound);
                    }
                }
                Ok(res)
            }
            None => {
                if !lower_bound_inclusive || !upper_bound_inclusive {
                    Err(Error::SingleVersionNotInclusive)
                } else {
                    let version = ComparableVersion::parse(process)?;
                    Ok(Self {
                        lower_bound: Some(version.clone()),
                        lower_bound_inclusive,
                        upper_bound: Some(version),
                        upper_bound_inclusive,
                    })
                }
            }
        }
    }
}
impl VersionRange {
    pub fn get_recommended(&self) -> Option<ComparableVersion> {
        self.recommended_version.clone()
    }
    pub fn contains_version(&self, version: &ComparableVersion) -> bool {
        for r in self.restrictions.iter() {
            if r.contains_version(version) {
                return true;
            }
        }
        false
    }
    pub fn parse(range: String) -> Result<Self> {
        let mut process = range;
        let mut restrictions: Vec<Restriction> = vec![];
        let mut recommended: Option<ComparableVersion> = None;
        let mut lower: Option<ComparableVersion> = None;
        let mut upper: Option<ComparableVersion> = None;
        while process.starts_with('[') || process.starts_with('(') {
            let index1 = process.index_of(']');
            let index2 = process.index_of(')');
            let index = match (index1, index2) {
                (None, None) => return Err(Error::UnboundRange),
                (None, Some(i)) => i,
                (Some(i), None) => i,
                (Some(i1), Some(i2)) => {
                    if i1 > i2 {
                        i2
                    } else {
                        i1
                    }
                }
            };
            let restr = Restriction::parse(process.substring(0, index + 1))?;
            if lower.is_none() {
                lower = restr.lower_bound.clone();
            }
            if upper.is_some()
                && (restr.lower_bound.is_none() || restr.lower_bound.value() < upper.value())
            {
                return Err(Error::RangeOverlap);
            }
            upper = restr.upper_bound.clone();
            restrictions.push(restr);
            process = process.substring(index + 1, process.length());
            if process.starts_with(',') {
                process = process.substring(1, process.length());
            }
        }
        if process.length() > 0 {
            if !restrictions.is_empty() {
                return Err(Error::IllegalVersionRange);
            } else {
                restrictions.push(Restriction::unrestricted());
                recommended = Some(ComparableVersion::parse(process)?);
            }
        }
        Ok(Self {
            restrictions,
            recommended_version: recommended,
        })
    }
}
