use std::str::FromStr;

use semver::VersionReq;

use crate::{
    errors::Error,
    maven::{restriction::VersionRange, version::ComparableVersion},
};

#[derive(Clone, Eq, Debug)]
pub struct Version {
    raw: String,
    maven: ComparableVersion,
    semver: Option<semver::Version>,
}

#[derive(Clone, Debug)]
pub struct VersionRestriction {
    raw: String,
    exact: Option<Version>,
    restriction: VersionRestrictionInner,
}

#[derive(Clone, Debug)]
enum VersionRestrictionInner {
    Semver(VersionReq),
    Maven(VersionRange),
}

impl VersionRestriction {
    pub fn parse(raw: String) -> Result<Self, Error> {
        if !raw.is_ascii() {
            return Err(Error::IllegalVersionRange);
        }
        let trimmed = raw.trim().to_string();
        let is_semver =
            trimmed.starts_with('=') || trimmed.starts_with('<') || trimmed.starts_with('>');
        let mut exact: Option<Version> = None;

        let inner = if is_semver {
            let semver = VersionReq::parse(trimmed.as_str())?;
            VersionRestrictionInner::Semver(semver)
        } else {
            let maven = VersionRange::parse(trimmed)?;
            if maven.restrictions.is_empty() {
                let recomm = maven.get_recommended().ok_or(Error::IllegalVersionRange)?;
                exact = Some(Version::parse(recomm.to_string())?);
            }

            VersionRestrictionInner::Maven(maven)
        };
        Ok(VersionRestriction {
            raw,
            exact,
            restriction: inner,
        })
    }

    pub fn contains_version(&self, version: &Version) -> bool {
        match &self.restriction {
            VersionRestrictionInner::Semver(semver) => {
                if let Some(sv) = &version.semver {
                    semver.matches(sv)
                } else {
                    false
                }
            }
            VersionRestrictionInner::Maven(maven) => maven.contains_version(&version.maven),
        }
    }

    pub fn is_exact_version(&self) -> bool {
        self.exact.is_some()
    }

    pub fn get_exact_version(&self) -> Option<Version> {
        self.exact.clone()
    }
}

impl FromStr for VersionRestriction {
    type Err = Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        Self::parse(s.to_string())
    }
}

impl ToString for VersionRestriction {
    fn to_string(&self) -> String {
        self.raw.clone()
    }
}

impl PartialEq for Version {
    fn eq(&self, other: &Self) -> bool {
        self.maven == other.maven
    }
}

impl PartialOrd for Version {
    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
        self.maven.partial_cmp(&other.maven)
    }
}

impl Ord for Version {
    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
        self.maven.cmp(&other.maven)
    }
}

impl Version {
    pub fn parse(raw: String) -> Result<Self, Error> {
        if raw.contains(char::is_whitespace) || !raw.is_ascii() {
            return Err(Error::InvalidVersion);
        }
        let maven = ComparableVersion::parse(raw.clone())?;
        let semver = semver::Version::parse(raw.as_str()).ok();

        Ok(Self { raw, maven, semver })
    }

    pub fn is_semver(&self) -> bool {
        self.semver.is_some()
    }
}

impl FromStr for Version {
    type Err = Error;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        Self::parse(s.to_string())
    }
}

impl ToString for Version {
    fn to_string(&self) -> String {
        self.raw.clone()
    }
}
